@@ -1,3 +1,4 @@
+import django
 from django.db import models
 from modeltree.query import ModelTreeQuerySet
 
@@ -11,4 +12,7 @@ class ModelTreeManager(models.Manager):
         return ModelTreeQuerySet(model=self.tree, using=self.db)
 
     def select(self, *args, **kwargs):
-        return self.get_query_set().select(*args, **kwargs)
+        if django.VERSION < (1, 6):
+            return self.get_query_set().select(*args, **kwargs)
+
+        return self.get_queryset().select(*args, **kwargs)
