@@ -1,3 +1,4 @@
+import django
 import inspect
 import warnings
 from django.db import models
@@ -107,6 +108,14 @@ class ModelTreeNode(object):
         else:
             return f.field.m2m_db_table()
 
+    @property
+    def m2m_related(self):
+        f = getattr(self.parent_model, self.accessor_name)
+        if self.reverse:
+            return f.related
+        else:
+            return f.field.related
+
     @property
     def m2m_field(self):
         f = getattr(self.parent_model, self.accessor_name)
@@ -124,62 +133,106 @@ class ModelTreeNode(object):
             return f.field.m2m_reverse_name()
 
     @property
-    def foreignkey_field(self):
+    def foreignkey_field_column(self):
         f = getattr(self.parent_model, self.accessor_name)
         if self.reverse:
             return f.related.field.column
         else:
             return f.field.column
 
+    @property
+    def foreignkey_field(self):
+        f = getattr(self.parent_model, self.accessor_name)
+        if self.reverse:
+            return f.related.field
+        else:
+            return f.field
+
+    def get_connection(self, lhs, table, lhs_col, col):
+        """
+        Creates a django version-independent connection tuple.
+
+        As of Django 1.6, the connection is now a 3-tuple instead of a
+        4-tuple where the third value is itself a tuple containing
+        columns to join on(for example, ((l_id1, r_id1), (l_id2, r_id2))).
+
+        More information on this see the comments here:
+            https://github.com/django/django/blob/9c487b5974ee7e7f196079611d7352364e8873ed/django/db/models/sql/query.py#L832-L858  # noqa
+        """
+        if django.VERSION < (1, 6):
+            return (lhs, table, lhs_col, col)
+        else:
+            if lhs_col is None and col is None:
+                return (lhs, table, None)
+
+            return (lhs, table, ((lhs_col, col),))
+
     def get_joins(self, **kwargs):
         """Returns a list of connections that need to be added to a
         QuerySet object that properly joins this model and the parent.
         """
         kwargs.setdefault('nullable', self.nullable)
-        kwargs.setdefault('outer_if_first', self.nullable)
+
+        if django.VERSION < (1, 7):
+            kwargs.setdefault('outer_if_first', self.nullable)
 
         joins = []
         # setup initial FROM clause
         copy = kwargs.copy()
-        copy['connection'] = (None, self.parent.db_table, None, None)
+        copy['connection'] = \
+            self.get_connection(None, self.parent.db_table, None, None)
         joins.append(copy)
 
-        # setup two connections for m2m
+        # Setup two connections for m2m.
         if self.relation == 'manytomany':
-            c1 = (
+            c1 = self.get_connection(
                 self.parent.db_table,
                 self.m2m_db_table,
                 self.parent.pk_column,
                 self.m2m_reverse_field if self.reverse else self.m2m_field,
             )
 
-            c2 = (
+            c2 = self.get_connection(
                 self.m2m_db_table,
                 self.db_table,
                 self.m2m_field if self.reverse else self.m2m_reverse_field,
                 self.pk_column,
             )
 
-            copy = kwargs.copy()
-            copy['connection'] = c1
-            joins.append(copy)
+            copy1 = kwargs.copy()
+            copy1['connection'] = c1
 
-            copy = kwargs.copy()
-            copy['connection'] = c2
-            joins.append(copy)
+            copy2 = kwargs.copy()
+            copy2['connection'] = c2
 
+            if django.VERSION >= (1, 6):
+                path = self.m2m_related.get_path_info()
+
+                copy1['join_field'] = path[0].join_field.field
+                copy2['join_field'] = path[1].join_field
+
+                # See also: join_field.get_joining_columns()
+
+            joins.append(copy1)
+            joins.append(copy2)
         else:
-            c1 = (
+            c1 = self.get_connection(
                 self.parent.db_table,
                 self.db_table,
                 self.parent.pk_column if self.reverse
-                else self.foreignkey_field,
-                self.foreignkey_field if self.reverse
+                else self.foreignkey_field_column,
+                self.foreignkey_field_column if self.reverse
                 else self.pk_column,
             )
 
             copy = kwargs.copy()
             copy['connection'] = c1
+
+            # Django 1.6 requires a join_field to be set when the lhs is NOT
+            # None so we need to set this field explicity here.
+            if django.VERSION >= (1, 6):
+                copy['join_field'] = self.foreignkey_field
+
             joins.append(copy)
 
         return joins
@@ -249,8 +302,19 @@ class ModelTree(object):
         sometimes it is necessary to exclude join paths. For example if there
         are three possible paths and one should never occur.
 
-        A modeltree config takes the `required_routes` and `excluded_routes`
-        which is a list of routes in the above format.
+        A modeltree config can have `required_routes` and `excluded_routes`
+        entries, which are lists of routes in the above format.
+
+        A required route is defined as follows: a join to the specified target
+        model is only allowed from the specified source model.  A model can
+        only be specified as a target once in the list of required routes.
+        Note that the use of the `symmetrical` property of a route
+        implicitly adds another route with target and source models swapped,
+        so a model can be a target either directly or indirectly.  A single
+        source model can participate in multiple required routes.
+
+        An excluded route is more obvious: joining from the specified source
+        model to the specified target model is not allowed.
 
     """
     def __init__(self, model=None, **kwargs):
@@ -286,13 +350,13 @@ class ModelTree(object):
         self.excluded_models = [self.get_model(label, local=False)
                                 for label in excluded_models]
 
-        # Build the routes are allowed/preferred
-        self._required_joins, self._required_join_fields = \
-            self._build_routes(required_routes)
+        # Build the routes that are allowed/preferred
+        self._required_joins = self._build_routes(
+            required_routes,
+            allow_redundant_targets=False)
 
         # Build the routes that are excluded
-        self._excluded_joins, self._excluded_join_fields = \
-            self._build_routes(excluded_routes)
+        self._excluded_joins = self._build_routes(excluded_routes)
 
         # cache each node relative their models
         self._nodes = {}
@@ -416,11 +480,19 @@ class ModelTree(object):
             model = self.root_model
         return model._meta.get_field_by_name(name)[0]
 
-    def _build_routes(self, routes):
-        "Routes provide a means of specifying JOINs between two tables."
+    def _build_routes(self, routes, allow_redundant_targets=True):
+        """Routes provide a means of specifying JOINs between two tables.
+
+        routes - a collection of dicts defining source->target mappings
+                 with optional `field` specifier and `symmetrical` attribute.
+
+        allow_redundant_targets - whether two routes in this collection
+                 are allowed to have the same target - this should NOT
+                 be allowed for required routes.
+        """
         routes = routes or ()
         joins = {}
-        join_fields = {}
+        targets_seen = set()
 
         for route in routes:
             if isinstance(route, dict):
@@ -456,20 +528,32 @@ class ModelTree(object):
                 if isinstance(field, RelatedObject):
                     field = field.field
 
-            # the `joins` hash defines pairs which are explicitly joined
-            # via the specified field
-            # if no field is defined, then the join field is implied or
-            # does not matter. the route is reduced to a straight lookup
-            joins[target] = source
+            if not allow_redundant_targets:
+                if target in targets_seen:
+                    raise ValueError('Model {0} cannot be the target of '
+                                     'more than one route in this list'
+                                     .format(target_label))
+                else:
+                    targets_seen.add(target)
+
+            # The `joins` hash defines pairs which are explicitly joined
+            # via the specified field.  If no field is defined, then the
+            # join field is implied or does not matter; the route is reduced
+            #  to a straight lookup.
+            joins[(source, target)] = field
+
             if symmetrical:
-                joins[source] = target
+                if not allow_redundant_targets:
+                    if source in targets_seen:
+                        raise ValueError('Model {0} cannot be the target of '
+                                         'more than one route in this list'
+                                         .format(source_label))
+                    else:
+                        targets_seen.add(source)
 
-            if field is not None:
-                join_fields[(source, target)] = field
-                if symmetrical:
-                    join_fields[(target, source)] = field
+                joins[(target, source)] = field
 
-        return joins, join_fields
+        return joins
 
     def _join_allowed(self, source, target, field=None):
         """Checks if the join between `source` and `target` via `field`
@@ -489,27 +573,23 @@ class ModelTree(object):
         if target == self.root_model:
             return False
 
-        # Check if the join is excluded via a specific field
-        if field and join in self._excluded_join_fields:
-            _field = self._excluded_join_fields[join]
-            if _field == field:
+        # Apply excluded joins if any
+        if join in self._excluded_joins:
+            _field = self._excluded_joins[join]
+            if not _field:
+                return False
+            elif _field and _field == field:
                 return False
 
-        # Model level..
-        elif source == self._excluded_joins.get(target):
-            return False
+        # Check if the join is allowed by a required rule
+        for (_source, _target), _field in self._required_joins.items():
+            if _target == target:
+                if _source != source:
+                    return False
 
-        # Check if the join is allowed
-        if target in self._required_joins:
-            _source = self._required_joins[target]
-            if _source != source:
-                return False
-
-            # If a field is supplied, check to see if the field is allowed
-            # for this join.
-            if field:
-                _field = self._required_join_fields.get(join)
-                if _field and _field != field:
+                # If a field is supplied, check to see if the field is allowed
+                # for this join.
+                if field and _field and _field != field:
                     return False
 
         return True
@@ -787,6 +867,7 @@ class ModelTree(object):
                 joins.extend(node.get_joins(**kwargs)[1:])
             else:
                 joins.extend(node.get_joins(**kwargs))
+
         return joins
 
     def query_string(self, model):
@@ -862,17 +943,35 @@ class ModelTree(object):
 
         aliases = []
 
-        for field in fields:
-            queryset, alias = self.add_joins(field.model, queryset, **kwargs)
-            aliases.append((alias, field.column))
+        for pair in fields:
+            if isinstance(pair, (list, tuple)):
+                model, field = pair
+            else:
+                field = pair
+                model = field.model
+
+            queryset, alias = self.add_joins(model, queryset, **kwargs)
+
+            col = (alias, field.column)
+
+            if django.VERSION >= (1, 6):
+                from django.db.models.sql.constants import SelectInfo
+
+                aliases.append(SelectInfo(col, field))
+            else:
+                aliases.append(col)
 
         if aliases:
             queryset.query.select = aliases
+
         return queryset
 
     def get_queryset(self):
         "Returns a QuerySet relative to the `root_model`."
-        return self.root_model._default_manager.get_query_set()
+        if django.VERSION < (1, 6):
+            return self.root_model._default_manager.get_query_set()
+
+        return self.root_model._default_manager.get_queryset()
 
 
 class LazyModelTrees(object):
@@ -897,7 +996,12 @@ class LazyModelTrees(object):
         return True
 
     def _get_model_label(self, model):
-        return '{0}.{1}'.format(model._meta.app_label, model._meta.module_name)
+        if django.VERSION < (1, 6):
+            model_name = model._meta.module_name
+        else:
+            model_name = model._meta.model_name
+
+        return '{0}.{1}'.format(model._meta.app_label, model_name)
 
     def _get_or_create(self, alias=None, **kwargs):
         # Echo back an existing modeltree
